#!/usr/bin/env python3
"""
Generate Grafana Dashboard JSON for vLLM Semantic Router
"""

import json


def create_stat_panel(title, expr, unit="short", x=0, y=0, w=6, h=6, panel_id=1):
    """Create a stat panel"""
    return {
        "datasource": {"type": "prometheus", "uid": "${DS_PROMETHEUS}"},
        "fieldConfig": {
            "defaults": {
                "color": {"mode": "thresholds"},
                "mappings": [],
                "thresholds": {
                    "mode": "absolute",
                    "steps": [{"color": "green", "value": None}],
                },
                "unit": unit,
            }
        },
        "gridPos": {"h": h, "w": w, "x": x, "y": y},
        "id": panel_id,
        "options": {
            "colorMode": "value",
            "graphMode": "area",
            "justifyMode": "auto",
            "orientation": "auto",
            "reduceOptions": {"calcs": ["lastNotNull"], "fields": "", "values": False},
            "textMode": "auto",
        },
        "pluginVersion": "11.5.1",
        "targets": [
            {
                "datasource": {"type": "prometheus", "uid": "${DS_PROMETHEUS}"},
                "expr": expr,
                "refId": "A",
            }
        ],
        "title": title,
        "type": "stat",
    }


def create_timeseries_panel(
    title, targets, x=0, y=0, w=12, h=8, panel_id=1, unit="short"
):
    """Create a time series panel"""
    return {
        "datasource": {"type": "prometheus", "uid": "${DS_PROMETHEUS}"},
        "fieldConfig": {
            "defaults": {
                "color": {"mode": "palette-classic"},
                "custom": {
                    "axisBorderShow": False,
                    "axisCenteredZero": False,
                    "axisColorMode": "text",
                    "axisLabel": "",
                    "axisPlacement": "auto",
                    "barAlignment": 0,
                    "drawStyle": "line",
                    "fillOpacity": 10,
                    "gradientMode": "none",
                    "hideFrom": {"tooltip": False, "viz": False, "legend": False},
                    "insertNulls": False,
                    "lineInterpolation": "linear",
                    "lineWidth": 1,
                    "pointSize": 5,
                    "scaleDistribution": {"type": "linear"},
                    "showPoints": "never",
                    "spanNulls": False,
                    "stacking": {"group": "A", "mode": "none"},
                    "thresholdsStyle": {"mode": "off"},
                },
                "mappings": [],
                "thresholds": {
                    "mode": "absolute",
                    "steps": [{"color": "green", "value": None}],
                },
                "unit": unit,
            }
        },
        "gridPos": {"h": h, "w": w, "x": x, "y": y},
        "id": panel_id,
        "options": {
            "legend": {
                "calcs": [],
                "displayMode": "list",
                "placement": "bottom",
                "showLegend": True,
            },
            "tooltip": {"mode": "multi", "sort": "none"},
        },
        "pluginVersion": "11.5.1",
        "targets": targets,
        "title": title,
        "type": "timeseries",
    }


def create_row_panel(title, y=0, panel_id=100):
    """Create a row panel"""
    return {
        "collapsed": False,
        "gridPos": {"h": 1, "w": 24, "x": 0, "y": y},
        "id": panel_id,
        "panels": [],
        "title": title,
        "type": "row",
    }


def create_target(expr, legend="", ref_id="A"):
    """Create a query target"""
    return {
        "datasource": {"type": "prometheus", "uid": "${DS_PROMETHEUS}"},
        "expr": expr,
        "legendFormat": legend,
        "refId": ref_id,
    }


def create_bar_chart_panel(
    title, targets, x=0, y=0, w=24, h=8, panel_id=1, unit="short"
):
    """Create a bar chart panel"""
    return {
        "datasource": {"type": "prometheus", "uid": "${DS_PROMETHEUS}"},
        "fieldConfig": {
            "defaults": {
                "color": {"mode": "palette-classic"},
                "custom": {
                    "axisBorderShow": False,
                    "axisCenteredZero": False,
                    "axisColorMode": "text",
                    "axisLabel": "",
                    "axisPlacement": "auto",
                    "barAlignment": 0,
                    "drawStyle": "bars",
                    "fillOpacity": 80,
                    "gradientMode": "none",
                    "hideFrom": {"tooltip": False, "viz": False, "legend": False},
                    "insertNulls": False,
                    "lineInterpolation": "linear",
                    "lineWidth": 1,
                    "pointSize": 5,
                    "scaleDistribution": {"type": "linear"},
                    "showPoints": "never",
                    "spanNulls": False,
                    "stacking": {"group": "A", "mode": "none"},
                    "thresholdsStyle": {"mode": "off"},
                },
                "mappings": [],
                "thresholds": {
                    "mode": "absolute",
                    "steps": [{"color": "green", "value": None}],
                },
                "unit": unit,
            }
        },
        "gridPos": {"h": h, "w": w, "x": x, "y": y},
        "id": panel_id,
        "options": {
            "legend": {
                "calcs": ["lastNotNull"],
                "displayMode": "table",
                "placement": "right",
                "showLegend": True,
            },
            "tooltip": {"mode": "multi", "sort": "desc"},
        },
        "pluginVersion": "11.5.1",
        "targets": targets,
        "title": title,
        "type": "timeseries",
    }


def generate_dashboard():
    """Generate the complete dashboard"""
    panels = []
    panel_id = 1
    y_pos = 0

    # ========== 1. Overall Request Metrics ==========
    panels.append(create_row_panel("Overall Request Metrics", y=y_pos, panel_id=100))
    y_pos += 1

    # Total Requests (increase over time range)
    panels.append(
        create_stat_panel(
            "Total Requests",
            "sum(increase(llm_model_requests_total[$__range]))",
            unit="short",
            x=0,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Average QPS (over time range)
    panels.append(
        create_stat_panel(
            "Average QPS",
            "sum(rate(llm_model_requests_total[$__range]))",
            unit="reqps",
            x=8,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Success Rate (over time range)
    panels.append(
        create_stat_panel(
            "Success Rate",
            "sum(increase(llm_model_requests_total[$__range])) / (sum(increase(llm_model_requests_total[$__range])) + (sum(increase(llm_request_errors_total[$__range])) or vector(0))) * 100",
            unit="percent",
            x=16,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1
    y_pos += 6

    # Request Count Trend
    panels.append(
        create_timeseries_panel(
            "Request Count Trend",
            [
                create_target(
                    "sum(rate(llm_model_requests_total[5m]))", "Requests/sec", "A"
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="reqps",
        )
    )
    panel_id += 1

    # Request Latency Trend
    panels.append(
        create_timeseries_panel(
            "Request Latency (P50/P95/P99)",
            [
                create_target(
                    "histogram_quantile(0.50, sum(rate(llm_model_completion_latency_seconds_bucket[5m])) by (le))",
                    "P50",
                    "A",
                ),
                create_target(
                    "histogram_quantile(0.95, sum(rate(llm_model_completion_latency_seconds_bucket[5m])) by (le))",
                    "P95",
                    "B",
                ),
                create_target(
                    "histogram_quantile(0.99, sum(rate(llm_model_completion_latency_seconds_bucket[5m])) by (le))",
                    "P99",
                    "C",
                ),
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="s",
        )
    )
    panel_id += 1
    y_pos += 8

    # ========== 2. Token Usage ==========
    panels.append(create_row_panel("LLM Token Usage", y=y_pos, panel_id=200))
    y_pos += 1

    # Total Tokens (increase over time range)
    panels.append(
        create_stat_panel(
            "Total Tokens",
            "sum(increase(llm_model_tokens_total[$__range]))",
            unit="short",
            x=0,
            y=y_pos,
            w=6,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Average Total Tokens/sec (over time range)
    panels.append(
        create_stat_panel(
            "Avg Tokens/sec",
            "sum(rate(llm_model_tokens_total[$__range]))",
            unit="tps",
            x=6,
            y=y_pos,
            w=6,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Average Prompt Tokens/sec (over time range)
    panels.append(
        create_stat_panel(
            "Avg Prompt Tokens/sec",
            "sum(rate(llm_model_prompt_tokens_total[$__range]))",
            unit="tps",
            x=12,
            y=y_pos,
            w=6,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Average Completion Tokens/sec (over time range)
    panels.append(
        create_stat_panel(
            "Avg Completion Tokens/sec",
            "sum(rate(llm_model_completion_tokens_total[$__range]))",
            unit="tps",
            x=18,
            y=y_pos,
            w=6,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1
    y_pos += 6

    # Token Usage Trend
    panels.append(
        create_timeseries_panel(
            "Token Usage Trend",
            [
                create_target("sum(rate(llm_model_tokens_total[5m]))", "Total", "A"),
                create_target(
                    "sum(rate(llm_model_prompt_tokens_total[5m]))", "Prompt", "B"
                ),
                create_target(
                    "sum(rate(llm_model_completion_tokens_total[5m]))",
                    "Completion",
                    "C",
                ),
            ],
            x=0,
            y=y_pos,
            w=24,
            h=8,
            panel_id=panel_id,
            unit="tps",
        )
    )
    panel_id += 1
    y_pos += 8

    # ========== 3. Signal Extraction ==========
    panels.append(create_row_panel("Signal Extraction", y=y_pos, panel_id=300))
    y_pos += 1

    # Signal Extraction Rate by Type
    panels.append(
        create_timeseries_panel(
            "Signal Extraction Rate by Type",
            [
                create_target(
                    "sum(rate(llm_signal_extraction_total[5m])) by (signal_type)",
                    "{{signal_type}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1

    # Signal Match Rate by Type
    panels.append(
        create_timeseries_panel(
            "Signal Match Rate by Type",
            [
                create_target(
                    "sum(rate(llm_signal_match_total[5m])) by (signal_type)",
                    "{{signal_type}}",
                    "A",
                )
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1
    y_pos += 8

    # Signal Extraction Latency P95
    panels.append(
        create_timeseries_panel(
            "Signal Extraction Latency (P95) by Type",
            [
                create_target(
                    "histogram_quantile(0.95, sum(rate(llm_signal_extraction_latency_seconds_bucket[5m])) by (le, signal_type))",
                    "{{signal_type}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="s",
        )
    )
    panel_id += 1

    # Top Matched Signals
    panels.append(
        create_timeseries_panel(
            "Top 10 Matched Signals",
            [
                create_target(
                    "topk(10, sum(rate(llm_signal_match_total[5m])) by (signal_name))",
                    "{{signal_name}}",
                    "A",
                )
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1
    y_pos += 8

    # ========== 4. Decision Matching ==========
    panels.append(create_row_panel("Decision Matching", y=y_pos, panel_id=400))
    y_pos += 1

    # Decision Evaluation Rate
    panels.append(
        create_stat_panel(
            "Decision Evaluation Rate",
            "rate(llm_decision_evaluation_total[5m])",
            unit="ops",
            x=0,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Decision Match Rate
    panels.append(
        create_stat_panel(
            "Decision Match Rate",
            "sum(rate(llm_decision_match_total[5m]))",
            unit="ops",
            x=8,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Decision Evaluation Latency P95
    panels.append(
        create_stat_panel(
            "Decision Evaluation Latency (P95)",
            "histogram_quantile(0.95, sum(rate(llm_decision_evaluation_latency_seconds_bucket[5m])) by (le))",
            unit="s",
            x=16,
            y=y_pos,
            w=8,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1
    y_pos += 6

    # Decision Match Trend by Decision
    panels.append(
        create_timeseries_panel(
            "Decision Match Trend by Decision",
            [
                create_target(
                    "sum(rate(llm_decision_match_total[5m])) by (decision_name)",
                    "{{decision_name}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1

    # Decision Confidence (P50) by Decision
    panels.append(
        create_timeseries_panel(
            "Decision Confidence (P50) by Decision",
            [
                create_target(
                    "histogram_quantile(0.5, sum(rate(llm_decision_confidence_bucket[5m])) by (le, decision_name))",
                    "{{decision_name}}",
                    "A",
                )
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="percentunit",
        )
    )
    panel_id += 1
    y_pos += 8

    # ========== 5. Model Distribution ==========
    panels.append(create_row_panel("Model Distribution", y=y_pos, panel_id=500))
    y_pos += 1

    # Model Request Count (Bar Chart - increase over time range)
    panels.append(
        create_bar_chart_panel(
            "Model Request Count",
            [
                create_target(
                    "sum(increase(llm_model_requests_total[$__range])) by (model)",
                    "{{model}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=24,
            h=8,
            panel_id=panel_id,
            unit="short",
        )
    )
    panel_id += 1
    y_pos += 8

    # Requests by Model
    panels.append(
        create_timeseries_panel(
            "Requests by Model",
            [
                create_target(
                    "sum(rate(llm_model_requests_total[5m])) by (model)",
                    "{{model}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="reqps",
        )
    )
    panel_id += 1

    # Errors by Model
    panels.append(
        create_timeseries_panel(
            "Errors by Model",
            [
                create_target(
                    "sum(rate(llm_request_errors_total[5m])) by (model, reason)",
                    "{{model}}-{{reason}}",
                    "A",
                )
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1
    y_pos += 8

    # TTFT by Model
    panels.append(
        create_timeseries_panel(
            "TTFT (Time to First Token) by Model - P95",
            [
                create_target(
                    "histogram_quantile(0.95, sum(rate(llm_model_ttft_seconds_bucket[5m])) by (le, model))",
                    "{{model}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="s",
        )
    )
    panel_id += 1

    # TPOT by Model
    panels.append(
        create_timeseries_panel(
            "TPOT (Time per Output Token) by Model - P95",
            [
                create_target(
                    "histogram_quantile(0.95, sum(rate(llm_model_tpot_seconds_bucket[5m])) by (le, model))",
                    "{{model}}",
                    "A",
                )
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="s",
        )
    )
    panel_id += 1
    y_pos += 8

    # ========== 6. Cache Plugin Metrics ==========
    panels.append(create_row_panel("Cache Plugin Metrics", y=y_pos, panel_id=600))
    y_pos += 1

    # Cache Hit Rate by Decision
    panels.append(
        create_timeseries_panel(
            "Cache Hit Rate by Decision",
            [
                create_target(
                    "sum(rate(llm_cache_plugin_hits_total[5m])) by (decision_name) / (sum(rate(llm_cache_plugin_hits_total[5m])) by (decision_name) + sum(rate(llm_cache_plugin_misses_total[5m])) by (decision_name)) * 100",
                    "{{decision_name}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="percent",
        )
    )
    panel_id += 1

    # Cache Hits/Misses by Decision
    panels.append(
        create_timeseries_panel(
            "Cache Hits/Misses by Decision",
            [
                create_target(
                    "sum(rate(llm_cache_plugin_hits_total[$__range])) by (decision_name)",
                    "{{decision_name}} - hits",
                    "A",
                ),
                create_target(
                    "sum(rate(llm_cache_plugin_misses_total[$__range])) by (decision_name)",
                    "{{decision_name}} - misses",
                    "B",
                ),
            ],
            x=12,
            y=y_pos,
            w=12,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1
    y_pos += 8

    # Current Cache Items by Backend
    panels.append(
        create_stat_panel(
            "Current Cache Items (Total)",
            "sum(llm_cache_entries_total)",
            unit="short",
            x=0,
            y=y_pos,
            w=6,
            h=6,
            panel_id=panel_id,
        )
    )
    panel_id += 1

    # Cache Items by Backend (Gauge)
    panels.append(
        create_timeseries_panel(
            "Cache Items by Backend",
            [
                create_target(
                    "llm_cache_entries_total",
                    "{{backend}}",
                    "A",
                )
            ],
            x=6,
            y=y_pos,
            w=18,
            h=6,
            panel_id=panel_id,
            unit="short",
        )
    )
    panel_id += 1
    y_pos += 6

    # Cache Item Operations Over Time (add/expired/evicted)
    panels.append(
        create_timeseries_panel(
            "Cache Item Operations (add/cleanup_expired/evict)",
            [
                create_target(
                    'sum(rate(llm_cache_operations_total{operation="add_entry", status="success"}[$__range])) by (backend)',
                    "{{backend}} - add",
                    "A",
                ),
                create_target(
                    'sum(rate(llm_cache_operations_total{operation="cleanup_expired", status="success"}[$__range])) by (backend)',
                    "{{backend}} - expired",
                    "B",
                ),
                create_target(
                    'sum(rate(llm_cache_operations_total{operation="evict", status="success"}[$__range])) by (backend)',
                    "{{backend}} - evicted",
                    "C",
                ),
            ],
            x=0,
            y=y_pos,
            w=24,
            h=8,
            panel_id=panel_id,
            unit="ops",
        )
    )
    panel_id += 1
    y_pos += 8

    # Cache Operation Latency
    panels.append(
        create_timeseries_panel(
            "Cache Operation Latency (p95)",
            [
                create_target(
                    "histogram_quantile(0.95, sum(rate(llm_cache_operation_duration_seconds_bucket[$__range])) by (backend, operation, le))",
                    "{{backend}} - {{operation}}",
                    "A",
                )
            ],
            x=0,
            y=y_pos,
            w=24,
            h=8,
            panel_id=panel_id,
            unit="s",
        )
    )
    panel_id += 1
    y_pos += 8

    return panels


def main():
    """Main function to generate and save the dashboard"""
    panels = generate_dashboard()

    dashboard = {
        "annotations": {"list": []},
        "editable": True,
        "fiscalYearStartMonth": 0,
        "graphTooltip": 1,
        "id": None,
        "links": [],
        "panels": panels,
        "refresh": "10s",
        "schemaVersion": 39,
        "tags": ["llm", "router", "semantic"],
        "templating": {
            "list": [
                {
                    "current": {
                        "selected": False,
                        "text": "Prometheus",
                        "value": "Prometheus",
                    },
                    "hide": 0,
                    "includeAll": False,
                    "label": "Datasource",
                    "multi": False,
                    "name": "DS_PROMETHEUS",
                    "options": [],
                    "query": "prometheus",
                    "refresh": 1,
                    "regex": "",
                    "skipUrlSync": False,
                    "type": "datasource",
                }
            ]
        },
        "time": {"from": "now-3h", "to": "now"},
        "timepicker": {},
        "timezone": "browser",
        "title": "vLLM Semantic Router Dashboard",
        "uid": "vllm-semantic-router",
        "version": 1,
        "weekStart": "",
    }

    # Save to file
    output_file = "llm-router-dashboard.serve.json"
    with open(output_file, "w") as f:
        json.dump(dashboard, f, indent=2)

    print(f"Dashboard generated successfully: {output_file}")
    print(f"Total panels: {len(panels)}")


if __name__ == "__main__":
    main()



import sys
import unittest
from pathlib import Path

import yaml

CLI_ROOT = Path(__file__).resolve().parents[1]
if str(CLI_ROOT) not in sys.path:
    sys.path.insert(0, str(CLI_ROOT))

from cli.parser import parse_user_config
from cli.validator import validate_user_config

TEMPLATE_PATH = CLI_ROOT / "cli" / "templates" / "config.template.yaml"


class TestConfigTemplate(unittest.TestCase):
    def test_template_is_lean_advanced_sample(self):
        with open(TEMPLATE_PATH, "r") as f:
            data = yaml.safe_load(f)

        self.assertEqual(data["version"], "v0.3")
        self.assertEqual(len(data["listeners"]), 1)
        self.assertEqual(
            data["providers"]["defaults"]["default_model"],
            "replace-with-your-model",
        )
        self.assertEqual(len(data["providers"]["models"]), 1)
        self.assertIn("backend_refs", data["providers"]["models"][0])
        self.assertEqual(len(data["routing"]["modelCards"]), 1)
        self.assertEqual(len(data["routing"]["decisions"]), 1)
        self.assertEqual(data["routing"]["decisions"][0]["name"], "default-route")
        self.assertEqual(data["routing"]["decisions"][0]["rules"]["conditions"], [])
        self.assertNotIn("signals", data.get("routing", {}))
        self.assertNotIn("memory", data)

    def test_template_excludes_legacy_demo_content(self):
        content = TEMPLATE_PATH.read_text()

        for legacy_name in ["math_keywords", "block_jailbreak", "remom_route"]:
            self.assertNotIn(
                legacy_name,
                content,
                f"template should not include legacy demo content: {legacy_name}",
            )

    def test_template_validates_without_legacy_merge_step(self):
        config_path = TEMPLATE_PATH

        user_config = parse_user_config(str(config_path))
        user_errors = validate_user_config(user_config)
        self.assertEqual([], user_errors)
        self.assertEqual(1, len(user_config.decisions))
        self.assertEqual("default-route", user_config.decisions[0].name)
        self.assertEqual("replace-with-your-model", user_config.providers.default_model)


if __name__ == "__main__":
    unittest.main()



from pathlib import Path

import yaml

from cli.bootstrap import (
    build_bootstrap_config,
    ensure_bootstrap_workspace,
    is_setup_mode_config,
)


def test_build_bootstrap_config_contains_setup_marker():
    config = build_bootstrap_config()

    assert config["version"] == "v0.3"
    assert config["listeners"][0]["port"] == 8899
    assert config["setup"]["mode"] is True
    assert config["setup"]["state"] == "bootstrap"


def test_ensure_bootstrap_workspace_creates_expected_files(tmp_path: Path):
    config_path = tmp_path / "config.yaml"

    result = ensure_bootstrap_workspace(config_path)

    assert result.created_config is True
    assert result.created_output_dir is True
    assert result.created_defaults is False
    assert result.setup_mode is True
    assert config_path.exists()
    assert (tmp_path / ".vllm-sr").exists()
    assert is_setup_mode_config(config_path) is True

    with open(config_path, "r") as f:
        data = yaml.safe_load(f)

    assert data["listeners"][0]["port"] == 8899
    assert data["setup"]["mode"] is True


def test_ensure_bootstrap_workspace_preserves_existing_setup_config(tmp_path: Path):
    config_path = tmp_path / "config.yaml"
    config_path.write_text(
        yaml.safe_dump(
            {
                "version": "v0.1",
                "listeners": [
                    {"name": "http-9999", "address": "0.0.0.0", "port": 9999}
                ],
                "setup": {"mode": True, "state": "bootstrap"},
            },
            sort_keys=False,
        )
    )

    result = ensure_bootstrap_workspace(config_path)

    assert result.created_config is False
    assert result.setup_mode is True

    with open(config_path, "r") as f:
        data = yaml.safe_load(f)

    assert data["listeners"][0]["port"] == 9999



from pathlib import Path

MAKEFILE_PATH = Path(__file__).resolve().parents[1] / "Makefile"


def test_makefile_uses_current_cli_config_commands() -> None:
    content = MAKEFILE_PATH.read_text(encoding="utf-8")

    assert "config-router: install" in content
    assert "config-envoy: install" in content
    assert "python -m cli.main config router --config $(CONFIG_FILE)" in content
    assert "python -m cli.main config envoy --config $(CONFIG_FILE)" in content
    assert "python -m cli.main validate --config $(CONFIG_FILE)" in content


def test_makefile_does_not_reference_removed_cli_commands() -> None:
    content = MAKEFILE_PATH.read_text(encoding="utf-8")

    forbidden = [
        "show-config: install",
        "python -m cli.main show-config",
        "generate: install",
        "python -m cli.main generate",
        "make show-config",
        "make generate     - Generate configurations",
    ]
    for needle in forbidden:
        assert (
            needle not in content
        ), f"removed CLI surface leaked into Makefile: {needle}"



import json
from pathlib import Path

PROJECT_ROOT = Path(__file__).resolve().parents[3]
SKILL_CATALOG_PATH = (
    PROJECT_ROOT / "dashboard" / "backend" / "config" / "openclaw-skills.json"
)
SKILLPACK_ROOT = PROJECT_ROOT / "dashboard" / "backend" / "skillpacks"


def test_openclaw_vsr_bridge_skill_is_cataloged_and_packaged() -> None:
    catalog = json.loads(SKILL_CATALOG_PATH.read_text(encoding="utf-8"))

    bridge_entry = next(
        (item for item in catalog if item.get("id") == "openclaw-vsr-bridge"),
        None,
    )

    assert bridge_entry is not None
    assert bridge_entry["builtin"] is True
    assert bridge_entry["category"] == "development"
    assert "curl" in bridge_entry.get("requires", [])
    assert "bash" in bridge_entry.get("requires", [])

    skill_doc = SKILLPACK_ROOT / "openclaw-vsr-bridge" / "SKILL.md"
    assert skill_doc.exists()

    skill_text = skill_doc.read_text(encoding="utf-8")
    assert "vllm-sr config import --from openclaw" in skill_text
    assert "--mode cli --runtime skip --no-launch" in skill_text



"""Container service and observability helpers for vLLM Semantic Router."""

import json
import os
import subprocess
from pathlib import Path

from cli.docker_images import get_fleet_sim_docker_image
from cli.docker_runtime import get_container_runtime
from cli.runtime_stack import RuntimeStackLayout, resolve_runtime_stack
from cli.utils import get_logger

log = get_logger(__name__)


def docker_container_status(container_name):
    """
    Get the status of a container.

    Returns:
        'running', 'exited', 'paused', or 'not found'
    """
    runtime = get_container_runtime()
    try:
        result = subprocess.run(
            [
                runtime,
                "ps",
                "-a",
                "--filter",
                f"name={container_name}",
                "--format",
                "{{.Status}}",
            ],
            capture_output=True,
            text=True,
            check=False,
        )
        status = result.stdout.strip()
        if not status:
            return "not found"
        if "Up" in status:
            return "running"
        if "Exited" in status:
            return "exited"
        if "Paused" in status:
            return "paused"
        return "unknown"
    except Exception as exc:
        log.error(f"Failed to get container status: {exc}")
        return "error"


def docker_stop_container(container_name):
    """Stop a container."""
    runtime = get_container_runtime()
    try:
        log.info(f"Stopping container: {container_name}")
        subprocess.run(
            [runtime, "stop", container_name], check=True, capture_output=True
        )
        log.info(f"Container stopped: {container_name}")
        return True
    except subprocess.CalledProcessError as exc:
        log.error(f"Failed to stop container: {exc}")
        return False


def docker_remove_container(container_name):
    """Remove a container."""
    runtime = get_container_runtime()
    try:
        log.info(f"Removing container: {container_name}")
        subprocess.run([runtime, "rm", container_name], check=True, capture_output=True)
        log.info(f"Container removed: {container_name}")
        return True
    except subprocess.CalledProcessError as exc:
        log.error(f"Failed to remove container: {exc}")
        return False


def docker_logs(container_name, follow=False, tail=None):
    """Stream logs from a container."""
    runtime = get_container_runtime()
    cmd = [runtime, "logs"]
    if follow:
        cmd.append("-f")
    if tail:
        cmd.extend(["--tail", str(tail)])
    cmd.append(container_name)

    try:
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError as exc:
        log.error(f"Failed to get logs: {exc}")
    except KeyboardInterrupt:
        log.info("Log streaming stopped")


def docker_logs_since(container_name, since_timestamp):
    """Get logs from a container since a specific timestamp."""
    runtime = get_container_runtime()
    cmd = [runtime, "logs", "--since", str(since_timestamp), container_name]

    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_exec(container_name, command):
    """Execute a command in a running container."""
    runtime = get_container_runtime()
    cmd = [runtime, "exec", container_name, *command]

    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_create_network(network_name):
    """Create a Docker network if it doesn't exist."""
    runtime = get_container_runtime()
    cmd = [
        runtime,
        "network",
        "ls",
        "--filter",
        f"name={network_name}",
        "--format",
        "{{.Name}}",
    ]
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        if network_name in result.stdout:
            log.debug(f"Network {network_name} already exists")
            return (0, "", "")
    except subprocess.CalledProcessError:
        pass

    cmd = [runtime, "network", "create", network_name]
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        log.info(f"Created network: {network_name}")
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_remove_network(network_name):
    """Remove a Docker network."""
    runtime = get_container_runtime()
    cmd = [runtime, "network", "rm", network_name]

    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_start_jaeger(
    network_name=None, stack_layout: RuntimeStackLayout | None = None
):
    """Start Jaeger container for distributed tracing."""
    runtime = get_container_runtime()
    stack_layout = stack_layout or resolve_runtime_stack()
    container_name = stack_layout.jaeger_container_name
    network_name = network_name or stack_layout.network_name
    _replace_existing_container(container_name)

    cmd = [
        runtime,
        "run",
        "-d",
        "--name",
        container_name,
        "--network",
        network_name,
        "-e",
        "COLLECTOR_OTLP_ENABLED=true",
        "-p",
        f"{stack_layout.jaeger_otlp_port}:4317",
        "-p",
        f"{stack_layout.jaeger_ui_port}:16686",
        "jaegertracing/all-in-one:latest",
    ]
    return _run_service_start(cmd, "Jaeger")


def docker_start_prometheus(
    network_name=None,
    config_dir=None,
    stack_layout: RuntimeStackLayout | None = None,
):
    """Start Prometheus container for metrics collection."""
    runtime = get_container_runtime()
    stack_layout = stack_layout or resolve_runtime_stack()
    container_name = stack_layout.prometheus_container_name
    network_name = network_name or stack_layout.network_name
    _replace_existing_container(container_name)

    config_dir = _ensure_hidden_config_dir(config_dir)
    prometheus_data_dir = os.path.join(config_dir, "prometheus-data")
    os.makedirs(prometheus_data_dir, exist_ok=True)

    prometheus_tsdb_dir = os.path.join(prometheus_data_dir, "data")
    os.makedirs(prometheus_tsdb_dir, exist_ok=True)
    try:
        os.chmod(prometheus_data_dir, 0o777)
        os.chmod(prometheus_tsdb_dir, 0o777)
    except Exception as exc:
        log.warning(f"Failed to set permissions on Prometheus data directory: {exc}")
        log.warning(
            "Prometheus may fail to start if it cannot write to the data directory"
        )

    prometheus_config_dir = os.path.join(config_dir, "prometheus-config")
    os.makedirs(prometheus_config_dir, exist_ok=True)
    prometheus_config = os.path.join(prometheus_config_dir, "prometheus.yaml")
    template_dir = os.path.join(os.path.dirname(__file__), "templates")
    _render_template_copy(
        os.path.join(template_dir, "prometheus.serve.yaml"),
        prometheus_config,
        stack_layout,
    )

    cmd = [
        runtime,
        "run",
        "-d",
        "--name",
        container_name,
        "--network",
        network_name,
        "-v",
        f"{os.path.abspath(prometheus_config)}:/etc/prometheus/prometheus.yaml:ro",
        "-v",
        f"{os.path.abspath(prometheus_data_dir)}:/prometheus",
        "-p",
        f"{stack_layout.prometheus_port}:9090",
        "prom/prometheus:v2.53.0",
        "--config.file=/etc/prometheus/prometheus.yaml",
        "--storage.tsdb.path=/prometheus/data",
        "--storage.tsdb.retention.time=15d",
    ]
    return _run_service_start(cmd, "Prometheus")


def docker_start_grafana(
    network_name=None,
    config_dir=None,
    stack_layout: RuntimeStackLayout | None = None,
):
    """Start Grafana container for visualization."""
    runtime = get_container_runtime()
    stack_layout = stack_layout or resolve_runtime_stack()
    container_name = stack_layout.grafana_container_name
    network_name = network_name or stack_layout.network_name
    _replace_existing_container(container_name)

    grafana_dir = os.path.join(_ensure_hidden_config_dir(config_dir), "grafana")
    os.makedirs(grafana_dir, exist_ok=True)

    template_dir = os.path.join(os.path.dirname(__file__), "templates")
    for filename in [
        "grafana.serve.ini",
        "grafana-datasource.serve.yaml",
        "grafana-datasource-jaeger.serve.yaml",
        "grafana-dashboard.serve.yaml",
        "llm-router-dashboard.serve.json",
    ]:
        _render_template_copy(
            os.path.join(template_dir, filename),
            os.path.join(grafana_dir, filename),
            stack_layout,
        )

    cmd = [
        runtime,
        "run",
        "-d",
        "--name",
        container_name,
        "--network",
        network_name,
        "-e",
        "GF_SECURITY_ADMIN_USER=admin",
        "-e",
        "GF_SECURITY_ADMIN_PASSWORD=admin",
        "-e",
        f"PROMETHEUS_URL={stack_layout.prometheus_container_name}:9090",
        "-v",
        f"{os.path.abspath(os.path.join(grafana_dir, 'grafana.serve.ini'))}:/etc/grafana/grafana.ini:ro",
        "-v",
        f"{os.path.abspath(os.path.join(grafana_dir, 'grafana-datasource.serve.yaml'))}:/etc/grafana/provisioning/datasources/datasource.yaml:ro",
        "-v",
        f"{os.path.abspath(os.path.join(grafana_dir, 'grafana-datasource-jaeger.serve.yaml'))}:/etc/grafana/provisioning/datasources/datasource_jaeger.yaml:ro",
        "-v",
        f"{os.path.abspath(os.path.join(grafana_dir, 'grafana-dashboard.serve.yaml'))}:/etc/grafana/provisioning/dashboards/dashboard.yaml:ro",
        "-v",
        f"{os.path.abspath(os.path.join(grafana_dir, 'llm-router-dashboard.serve.json'))}:/etc/grafana/provisioning/dashboards/llm-router-dashboard.json:ro",
        "-p",
        f"{stack_layout.grafana_port}:3000",
        "grafana/grafana:11.5.1",
    ]
    return _run_service_start(cmd, "Grafana")


def docker_start_fleet_sim(
    *,
    image: str | None = None,
    pull_policy: str | None = None,
    network_name: str | None = None,
    config_dir: str | None = None,
    stack_layout: RuntimeStackLayout | None = None,
):
    """Start the vllm-sr-sim sidecar container."""
    runtime = get_container_runtime()
    stack_layout = stack_layout or resolve_runtime_stack()
    container_name = stack_layout.fleet_sim_container_name
    network_name = network_name or stack_layout.network_name
    sim_image = get_fleet_sim_docker_image(image=image, pull_policy=pull_policy)
    _replace_existing_container(container_name)

    sim_state_dir = os.path.join(
        _ensure_hidden_config_dir(config_dir), "fleet-sim-state"
    )
    os.makedirs(sim_state_dir, exist_ok=True)

    cmd = [
        runtime,
        "run",
        "-d",
        "--name",
        container_name,
        "--network",
        network_name,
        "-e",
        "PYTHONUNBUFFERED=1",
        "-e",
        "VLLM_SR_SIM_STATE_DIR=/state",
        "-v",
        f"{os.path.abspath(sim_state_dir)}:/state",
        "-p",
        f"{stack_layout.fleet_sim_port}:8000",
        sim_image,
    ]
    return _run_service_start(cmd, "vllm-sr-sim")


def docker_network_disconnect(network_name, container_name):
    """Disconnect a container from a Docker network."""
    runtime = get_container_runtime()
    cmd = [runtime, "network", "disconnect", network_name, container_name]
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_network_connect(network_name, container_name):
    """Connect a container to a Docker network (idempotent)."""
    runtime = get_container_runtime()
    cmd = [runtime, "network", "connect", network_name, container_name]
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        if "already" in (exc.stderr or "").lower():
            return (0, exc.stdout, exc.stderr)
        return (exc.returncode, exc.stdout, exc.stderr)


def docker_start_container(container_name):
    """Start a stopped container."""
    runtime = get_container_runtime()
    try:
        log.info(f"Starting container: {container_name}")
        subprocess.run(
            [runtime, "start", container_name], check=True, capture_output=True
        )
        log.info(f"✓ Container started: {container_name}")
        return True
    except subprocess.CalledProcessError as exc:
        log.error(f"Failed to start container: {exc}")
        return False


def load_openclaw_registry(data_dir):
    """Load OpenClaw container entries from containers.json."""
    registry_path = os.path.join(data_dir, "containers.json")
    if not os.path.exists(registry_path):
        return []
    try:
        with open(registry_path) as handle:
            return json.load(handle)
    except (json.JSONDecodeError, OSError) as exc:
        log.warning(f"Failed to load OpenClaw registry: {exc}")
        return []


def _replace_existing_container(container_name):
    status = docker_container_status(container_name)
    if status != "not found":
        log.info(f"{container_name} already exists (status: {status}), cleaning up...")
        docker_stop_container(container_name)
        docker_remove_container(container_name)


def _ensure_hidden_config_dir(config_dir):
    if config_dir is None:
        config_dir = os.path.join(os.getcwd(), ".vllm-sr")
    else:
        config_dir = os.path.join(config_dir, ".vllm-sr")
    os.makedirs(config_dir, exist_ok=True)
    return config_dir


def _render_template_copy(
    source_path: str, destination_path: str, stack_layout: RuntimeStackLayout
) -> None:
    source = Path(source_path)
    destination = Path(destination_path)
    rendered = render_observability_template(source.read_text(), stack_layout)
    destination.write_text(rendered, encoding="utf-8")


def render_observability_template(
    template_text: str, stack_layout: RuntimeStackLayout
) -> str:
    replacements = {
        "vllm-sr-container": stack_layout.container_name,
        "vllm-sr-prometheus": stack_layout.prometheus_container_name,
        "vllm-sr-jaeger": stack_layout.jaeger_container_name,
    }
    rendered = template_text
    for source, target in replacements.items():
        rendered = rendered.replace(source, target)
    return rendered


def _run_service_start(cmd, service_name):
    log.info(f"Starting {service_name} container...")
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return (0, result.stdout, result.stderr)
    except subprocess.CalledProcessError as exc:
        return (exc.returncode, exc.stdout, exc.stderr)



"""vLLM Semantic Router CLI package."""

from __future__ import annotations

import re
from importlib.metadata import PackageNotFoundError, version
from pathlib import Path

_PYPROJECT_VERSION_PATTERN = re.compile(
    r'^version = "(?P<version>[^"]+)"$', re.MULTILINE
)


def _load_version() -> str:
    pyproject_path = Path(__file__).resolve().parents[1] / "pyproject.toml"
    try:
        match = _PYPROJECT_VERSION_PATTERN.search(
            pyproject_path.read_text(encoding="utf-8")
        )
    except FileNotFoundError:
        match = None
    if match is not None:
        return match.group("version")

    try:
        return version("vllm-sr")
    except PackageNotFoundError:
        return "unknown"


__version__ = _load_version()
